<?php

namespace App\Http\Controllers;

use App\Models\Patient;
use App\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Gate;
use App\Models\Clinic;
use App\Models\PatientFile;
use Illuminate\Support\Facades\Mail;
use Carbon\Carbon;

class PatientController extends Controller
{

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
    }
    
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if($request->ajax())
        {
            $patients = DB::table('patients')
                ->select('patients.id', 'patients.name')
                ->leftJoin('clinics as c', 'c.id', '=', 'patients.clinic_id')
                ->where('c.id', $request->clinic);
            if(Gate::allows('isClinic'))
            {
                $patients = $patients->where('c.user_id', '=', Auth::user()->id);
            }
                            
            if(!empty($request->search))
                $patients = $patients->where('patients.name', 'like', '%'.$request->search.'%');
            if(!empty($request->sort))
            {
                $ar = explode(',', $request->sort);
                $patients = $patients->orderBy($ar[0], $ar[1]);
            }
            
            $patients = $patients->get();
            
            return response()->json(['patients'=>$patients]);
        }
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        if($request->ajax())
        {
            DB::beginTransaction();
            try {
                if(!empty($request->patient_email))
                {
                    if(User::where('email', $request->patient_email)->count())
                        return response()->json(['status'=>0, 'msg'=>'This email is already registered']);
                    $user = new User();
                    $user->name = $request->patient_name;
                    $user->email = $request->patient_email;
                    $password = str_random(8);
                    $user->password = Hash::make($password);
                    $user->user_type = 'patient';
                    $user->save();
                }
                
                $patient = new Patient();
                $patient->name = $request->patient_name;
                $patient->age = $request->patient_age;
                $patient->email = $request->patient_email;
                if(!empty($request->patient_email))
                    $patient->user()->associate($user);
                $clinic = \App\Models\Clinic::find($request->clinic);
                $patient->clinic()->associate($clinic);
                $patient->save();
    
                DB::commit();
    
                // Move email sending outside the transaction
                $this->sendEmails($patient, $user ?? null, $password ?? null);
    
                return response()->json(['patient'=>$patient, 'msg'=>'Patient Folder Created Successfully']);
            } catch (\Exception $e) {
                DB::rollBack();
                return response()->json(['status'=>0, 'msg'=>'Error creating patient: '.$e->getMessage()]);
            }
        }
    }
    
    private function sendEmails($patient, $user = null, $password = null)
    {
        try {
            Mail::to('service3d@labbellomo.com')
                ->bcc('laboratoirebellomo@gmail.com')
                ->queue(new \App\Mail\PatientCreated($patient));
    
            if ($user && $password) {
                $user->notify(new \App\Notifications\ClinicCreated($user, $password));
            }
        } catch (\Exception $e) {
            // Log the error, but don't interrupt the patient creation process
            \Log::error('Failed to send email: '.$e->getMessage());
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Patient  $patient
     * @return \Illuminate\Http\Response
     */
    public function show(Patient $patient, Request $request)
    {
        if($request->ajax())
        {
            return response()->json(['patient'=>$patient]);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Patient  $patient
     * @return \Illuminate\Http\Response
     */
    public function edit(Patient $patient)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Patient  $patient
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Patient $patient)
    {
        $this->checkAuth($patient->clinic);
        $patient->name = $request->patient_name;
        $patient->age = $request->patient_age;
        $patient->email = $request->patient_email;
        $patient->save();
        return response()->json(['patient'=>$patient, 'msg'=>'patient updated successfully']);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Patient  $patient
     * @return \Illuminate\Http\Response
     */
    public function destroy(Patient $patient)
    {
        $this->checkAuth($patient->clinic);
        User::where('email', $patient->email)->delete();
        $patient->delete();
        return response()->json(['msg'=>'patient deleted successfully']);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function createFile(Patient $patient)
    {
        $this->checkAuth($patient->clinic);
        return view('patients/files/create', ['patient'=>$patient]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function storeFile(Request $request)
    {
        $patient = Patient::find($request->patient);
        $this->checkAuth($patient->clinic);
        $patientFile = new PatientFile();
        $patientFile->date_needed = $request->date_needed;
        $patientFile->note = $request->notes;
        $file = $request->file('files');
        if ($file == "") {
            $url = "";
        } else {
            $ext = $file->getClientOriginalExtension();
            $name = pathinfo($file->getClientOriginalName(), PATHINFO_FILENAME);
            $url = $name . "-" . date("Y"."m"."d") . "." . $ext;
            //dd($url);
            $file->storeAs('patients/files/', $url, 'public');
            $patientFile->file =$url;
        }
        $patientFile->patient()->associate($patient);
        $patientFile->user_id = Auth::user()->id;
        //dd($deletionDate);
        $patientFile->save();
        $deletionDate = Carbon::createFromFormat('Y-m-d H:i:s', $patientFile->created_at)->addMonths(6);
        $patientFile->date_deletion = $deletionDate;
        $patientFile->save();

        // Envoi d'e-mail avec gestion des erreurs
        try {
            Mail::to(['service3d@labbellomo.com', $patientFile->patient->clinic->email])
                ->bcc('laboratoirebellomo@gmail.com')
                ->send(new \App\Mail\PatientFileCreated($patientFile));
            $request->session()->flash('alert-success', 'Patient file added successfully and email sent.');
        } catch (\Exception $e) {
            \Log::error('Failed to send email: '.$e->getMessage());
            $request->session()->flash('alert-danger', 'Patient file added but failed to send email.');
        }

        return redirect()->back();
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\PatientFile  $patientFile
     * @return \Illuminate\Http\Response
     */
    public function editFile(PatientFile $patientFile, Request $request)
    {
        if($request->ajax())
        {
            $patientFile->formattedDate = date('Y-m-d', strtotime($patientFile->date_needed));
            // dd($patientFile->formattedDate);
            return response()->json(['patientFile'=>$patientFile]);
        }
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Patient  $patient
     * @return \Illuminate\Http\Response
     */
    public function updateFile(Request $request, PatientFile $patientFile)
    {
        //dd($request);
        $this->checkAuth($patientFile->patient->clinic);
        $patientFile->date_needed = $request->date_needed;
        $patientFile->note = $request->notes;
        $file = $request->file('files');
        if ($file == "") {
            $url = $patientFile->file;
        } else {
            $ext = $file->getClientOriginalExtension();
            $name = pathinfo($file->getClientOriginalName(), PATHINFO_FILENAME);
            $url = $name . "-" . date("Y"."m"."d") . "." . $ext;
            //dd($url);
            $file->storeAs('patients/files/', $url, 'public');
            $patientFile->file =$url;
        }
        $patientFile->user_id = Auth::user()->id;
        $patientFile->save();
        $request->session()->flash('alert-success', 'Patient file updated successfully');
        return redirect()->back();
    }

    public function deleteFile(PatientFile $patientFile)
    {
        $this->checkAuth($patientFile->patient->clinic);
        $patientFile->delete();
        session()->flash('alert-success', 'Patient file deleted successfully');
        return redirect()->back();
    }
    
    private function checkAuth($clinic)
    {
        if (Gate::denies('isAdmin') && $clinic->user_id != Auth::user()->id) {
            abort(403, 'Unauthorized action.');
        }
    }
}
